#!/bin/sh
source /etc/event_functions
NET_STATUS="/var/run/netstatus"


SEND_FILE="/var/run/test.txt"
UMOUNT="/bin/umount"

SMTPS_CONF="/etc/conf.d/smtps.conf"
pidfile="/var/run/smtps.pid"
pemfile="/etc/conf.d/privatekey.pem"
certfile="/etc/conf.d/publickey.cert"
HTTPS_CLIENT_CONF="../conf.d/usrdef/https_client.conf"
HTTPS_CLIENT_PID_FILE="/var/run/httpsclient.pid"

# program list
HTTP_CLIENT_PROG="/usr/sbin/httpclient"
STUNNEL_PROG="/usr/sbin/stunnel"
FTPMPUT_PROG="/usr/bin/ftpmput"
SMTPC=$(which smtpclient)
CURL_PROG=$(which curl)
FTPS_PROG=$(which ftps)
TINYXMLPARSER=$(which tinyxmlparser)

[ -e ${CURL_PROG} ] && cURL="true"
getEncParam() {
    if [ "$REQUEST_METHOD" = "POST" ]; then
        strparam=$(cat $stdin | cut -c1-$CONTENT_LENGTH)
    else
        strparam=$QUERY_STRING
    fi

    echo "${strparam}"
}

decParam() {
    input=$1
    echo $(except_decode.sh ${input})
}

parseParam() {
    strparam=$1
    type=$(echo "$strparam" | sed -n 's/^.*type=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    address=$(echo "$strparam" | sed -n 's/^.*address=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    username=$(echo "$strparam" | sed -n 's/^.*username=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    passwd=$(echo "$strparam" | sed -n 's/^.*passwd=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    port=$(echo "$strparam" | sed -n 's/^.*port=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    sslmode=$(echo "$strparam" | sed -n 's/^.*sslmode=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    passive=$(echo "$strparam" | sed -n 's/^.*passive=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    location=$(echo "$strparam" | sed -n 's/^.*location=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    senderemail=$(echo "$strparam" | sed -n 's/^.*senderemail=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    recipientemail=$(echo "$strparam" | sed -n 's/^.*recipientemail=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    workgroup=$(echo "$strparam" | sed -n 's/^.*workgroup=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    url=$(echo "$strparam" | sed -n 's/^.*url=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    resType=$(echo "$strparam" | sed -n 's/^.*response=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    refTarget=$(echo "$strparam" | sed -n 's/^.*reftarget=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    refPasswdByIndex=$(echo "$strparam" | sed -n 's/^.*refpasswdbyindex=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")

    [ -z "$refTarget" ] && refTarget="reportpush"
}

preprocess_http_url() {
    if [ "$type" = "https" ]; then
        sslmode="true"
    else
        sslmode="false"
    fi

    target_url="${url#${type}://}"
    url="/${target_url#*/}"
    address_port="${target_url%%/*}"

    if [ "$(echo ${address_port} | sed -e 's/^\(.\{1\}\).*$/\1/')" == "[" ]; then
        if [ "$(echo ${address_port} | sed -e 's/\(^.*\)\(.$\)/\2/')" == "]" ]; then
            address="$(echo ${address_port} | sed 's/^.\(.*\).$/\1/')"
            port=80
        else
            targetUrl="$(echo ${address_port} | sed 's/\(.*\):/\1 /')"
            address="$(echo ${address_port} | cut -d ' ' -f 1 | sed 's/^.\(.*\).$/\1/')"
            port="$(echo ${address_port} | cut -d ' ' -f 2)"
        fi

        use_ipv6="v6"
    else
        address="${address_port%%:*}"
        if [ "${address}" != "${address_port}" ]; then
            port="${address_port#${address}:}"
        else
            port=80
        fi
    fi
}

parse_data_from_config() {
    ref_target=$1 && pass_index=$2 && ref_file=$3

    case "${ref_target}" in
    "validation")
        svrType="ftp"
        svrAddr="$(${TINYXMLPARSER} -x /Validation/i${pass_index}/FTP/Address -f ${ref_file})"
        svrPort="$(${TINYXMLPARSER} -x /Validation/i${pass_index}/FTP/Port -f ${ref_file})"
        svrUser="$(${TINYXMLPARSER} -x /Validation/i${pass_index}/FTP/User -f ${ref_file})"
        svrPwd="$(${TINYXMLPARSER} -x /Validation/i${pass_index}/FTP/Password -f ${ref_file})"
        ;;
    "reportpush")
        svrType="$(${TINYXMLPARSER} -x /reportpush/i${pass_index}/servertypeselector -f ${ref_file})"
        svrAddr="$(${TINYXMLPARSER} -x /reportpush/i${pass_index}/url -f ${ref_file})"
        svrPort="$(${TINYXMLPARSER} -x /reportpush/i${pass_index}/port -f ${ref_file})"
        svrUser="$(${TINYXMLPARSER} -x /reportpush/i${pass_index}/usr -f ${ref_file})"
        svrPwd="$(${TINYXMLPARSER} -x /reportpush/i${pass_index}/pwd -f ${ref_file})"
        ;;
    esac
}

update_password() {
    pkgPath="$(dirname $(pwd))"

    if [ "${refTarget}" = "validation" ]; then
        config="${pkgPath}/conf.d/vca/config_${refTarget}.xml"
    else
        config="${pkgPath}/conf.d/vca/config_vca_${refTarget}.xml"
    fi

    parse_data_from_config ${refTarget} ${refPasswdByIndex} ${config}

    if [ "${svrType}" == "${type}" -a "${svrAddr}" == "${address}" -a "${svrPort}" == "${port}" -a "${svrUser}" == "${username}" ]; then
        passwd=${svrPwd}
    else
        resMsg="invalidPasswdRefer"
    fi

}

get_request_parameter() {
    strparam=$(getEncParam)
    strparam=$(decParam ${strparam})
    parseParam ${strparam}

    if [ "$type" = "http" -o "$type" = "https" ]; then
        preprocess_http_url
    fi

    if [ -n "${refPasswdByIndex}" -a -z "${passwd}" ]; then
        update_password
    fi
}
checkNetStatus() {
    if [ ! -f "${NET_STATUS}" ]; then
        echo "false"
    else
        echo "true"
    fi
}

generate_https_necessary_file() {
    gencert self >/dev/null
    chmod 600 /etc/conf.d/privatekey.pem
    gencert certinfo >/dev/null
}

generate_test_file() {
    if [ ! -f "$SEND_FILE" ]; then
        echo "[NOTIFICATION]The Result of Server Test of Your IP Camera" >"$SEND_FILE"
    fi
}

update_stunnel_config() {
    address_new=$1
    port_new=$2
    config=$3

    temp_file="stunnel_config.temp"
    new_address="$address_new:$port_new"
    org_address=$(grep "connect" $config | sed -n 's/connect = //p')
    if [ "$new_address" != "$org_address" ]; then
        grep -v "connect" $config >$temp_file
        echo "connect = $new_address" >>$temp_file
        mv $temp_file $config
    fi
}

prepare_protocol_over_stunnel() {
    config_file=$1
    address=$2
    port=$3

    if [ -f "$config_file" ]; then
        update_stunnel_config $address $port $config_file

        if [ -f $pemfile ] && [ -f $certfile ]; then
            $($STUNNEL_PROG $config_file)
            if [ "$?" != "0" ]; then
                generate_https_necessary_file
                $($STUNNEL_PROG $config_file)
            fi
        else
            generate_https_necessary_file
            $($STUNNEL_PROG $config_file)
        fi
    fi
}

emailSvr() {
    exitStatus=0

    if [ -n "$address" ] && [ -n "$recipientemail" ]; then
        title="[NOTIFICATION]The Result of Server Test of Your IP Camera"
        mime_type="-M 2"

        if [ -n "$username" ]; then
            auth="-U ${username} -p ${passwd}"
        else
            auth=""
        fi

        if [ "$sslmode" = "1" ]; then
            prepare_protocol_over_stunnel ${SMTPS_CONF} ${address} ${port}
            $SMTPC -s "$title" $mime_type -b $SEND_FILE -f "$senderemail" -S "127.0.0.1" -P "25" $auth "$recipientemail" $COS_PRIORITY_OPT $DSCP_PRIORITY_OPT
        else
            $SMTPC -s "$title" $mime_type -b $SEND_FILE -f "$senderemail" -S "$address" -P $port $auth "$recipientemail" $COS_PRIORITY_OPT $DSCP_PRIORITY_OPT
        fi

        if [ "$?" = "0" ]; then
            resMsg="sendEmailSuccess"
        else
            resMsg="sendEmailFail"
            exitStatus=1
        fi

        if [ "$sslmode" = "1" ]; then
            kill -9 $(cat $pidfile)
        fi
    else
        resMsg="errEmailAddr"
        exitStatus=1
    fi
}

ftpSvr() {
    exitStatus=0

    if [ -n "$address" ]; then
        if [ -n "$username" ]; then
            auth="-u $username -p $passwd"
        else
            auth=""
        fi

        if [ ! -n "$port" ]; then
            port="21"
        fi

        if [ "$passive" = "1" ]; then
            mode=""
        else
            . $NET_STATUS
            mode="-a $IP_ADDR"
        fi

        if [ -z $location ]; then
            DIR="."
        else
            DIR="$location"
        fi

        if [ -x ${FTPS_PROG} ]; then
            ${FTPS_PROG} $mode -t 60 $auth -P $port $address "$DIR" $SEND_FILE $COS_PRIORITY_OPT $DSCP_PRIORITY_OPT >/dev/null
        else
            ${FTPMPUT_PROG} $mode -t 60 $auth -P $port $address "$DIR" $SEND_FILE $COS_PRIORITY_OPT $DSCP_PRIORITY_OPT >/dev/null
        fi

        if [ "$?" = "0" ]; then
            resMsg="sendFtpSuccess"
        else
            resMsg="sendFtpFail"
            exitStatus=1
        fi
    else
        resMsg="errFtpAddr"
        exitStatus=1
    fi
}

httpSvr() {
    exitStatus=0

    if [ -n "$address" ]; then

        if [ "${sslmode}" = "true" ] && [ "${cURL}" != "true" ]; then
            prepare_protocol_over_stunnel ${HTTPS_CLIENT_CONF} ${address} ${port}
            address=127.0.0.1
            port=26
            use_ipv6=""
        fi

        if [ "${cURL}" = "true" ]; then
            [ "${use_ipv6}" = "v6" ] && address="[${address}]"
            http_ret_code=$("${CURL_PROG}" -s -o "/dev/null" -w "%{http_code}" "${type}://${address}:${port}${url}" -u "${username}:${passwd}" -F "data=@${SEND_FILE}" -k --connect-timeout 25)
        else
            $HTTP_CLIENT_PROG -M UPLOAD -t Text -file ${SEND_FILE} -S ${address} ${port} "${username}" "${passwd}" ${use_ipv6} -U "${url}" >/dev/null
        fi

        if [ "$?" = "0" -a "${cURL}" = "" -o "${cURL}" = "true" -a "${http_ret_code}" = "200" ]; then
            resMsg="sendHttpSuccess"
        else
            resMsg="sendHttpFail"
            exitStatus=1
        fi

        [ -f "${HTTPS_CLIENT_PID_FILE}" ] && kill -9 "$(cat ${HTTPS_CLIENT_PID_FILE})" && rm -f ${HTTPS_CLIENT_PID_FILE}
    else
        resMsg="errHttpAddr"
        exitStatus=1
    fi
}

nasSvr() {
    exitStatus=0
    smb_path="$(echo $location | sed -e 's#[\]#/#g')"
    tmp_path=${smb_path#//}
    smb_host=${tmp_path%%/*}
    tmp_path=${smb_path#//*/}
    smb_sharefolder=${tmp_path%%/*}
    smb_childfolder=${smb_path#//$smb_host/$smb_sharefolder}
    smb_path1="//${smb_host}/${smb_sharefolder}"

    destination_path="/mnt/auto/i99"
    if [ -n "$username" ]; then
        if [ -n "$workgroup" ]; then
            parameter="sec=ntlm,username=$username,password=$passwd,workgroup=$workgroup"
        else
            parameter="sec=ntlm,username=$username,password=$passwd"
        fi
    else
        parameter="sec=none"
    fi
    echo "i99 -fstype=cifs,$parameter :$smb_path1" >>/mnt/flash/etc/auto.conf
    cd $destination_path

    if [ "$?" = "0" ]; then
        cp $SEND_FILE $destination_path
        if [ "$?" = "0" ]; then
            resMsg="mountNasSuccess"
        else
            resMsg="nasDenied"
            exitStatus=1
        fi

        sleep 1
        cd /mnt/ramdisk
        $UMOUNT $destination_path
    else
        resMsg="mountNasFail"
        exitStatus=1
    fi

    sed -i '/i99 -fstype=cifs/d' /mnt/flash/etc/auto.conf
}

sdCard() {
    exitStatus=0
    destination_folder="/mnt/auto/CF/test"

    if [ ! -d "$destination_folder" ]; then
        mkdir $destination_folder
    fi

    if [ -d "$destination_folder" ]; then
        cp $SEND_FILE $destination_folder

        if [ "$?" = "0" ]; then
            resMsg="storeSdCardSuccess"
            rm $SEND_FILE
        fi
    else
        resMsg="storeSdCardFail"
        exitStatus=1
    fi
}

send_request_to_server() {
    if [ -z "$resMsg" ]; then
        if [ "$(checkNetStatus)" == "true" ]; then
            generate_test_file
            case "$type" in
            "email")
                emailSvr
                ;;
            "ftp")
                ftpSvr
                ;;
            "http" | "https")
                httpSvr
                ;;
            "ns")
                nasSvr
                ;;
            "sdcard")
                sdCard
                ;;
            esac
        else
            resMsg="checkNetFail"
        fi
    fi
    echo "$resMsg"
}
resHtmlHead()
{
    echo -ne "Content-type: text/html\r\n"
    echo -ne "\r\n"
    echo -ne "<html>\n
              <head>\n
              <meta http-equiv='Content-Type' content='text/html; charset=utf-8' />\n
              <script type='text/javascript' src='/include/common.js'></script>\n
              </head>\n
              <body>\n"
}

resHtmlTail()
{
    echo "</body></html>"
}

transHtml()
{
    echo -e "<script type="text/javascript">document.write(translator('$1'))</script>"
}

resHtml()
{
    respnse_msg=$1
    resHtmlHead
    case "${respnse_msg}" in
        "checkNetFail")
	        transHtml "please_check_internet_connection"
            ;;
        "sendEmailFail")
            transHtml "error_in_sending_email"
            ;;
        "sendEmailSuccess")
            transHtml "the_email_has_been_sent_successfully"
            connect_success=1
            ;;
        "errEmailAddr")
            transHtml "please_define_mail_server_location"
            ;;
        "sendFtpSuccess")
            transHtml "ftp_transmission_successfully"
            connect_success=1
            ;;
        "sendFtpFail")
            transHtml "ftp_transmission_failed"
            ;;
        "errFtpAddr")
            transHtml "please_define_ftp_address"
            ;;
        "sendHttpSuccess")
            transHtml "http_transmission_successfully"
            connect_success=1
            ;;
        "sendHttpFail")
            transHtml "http_transmission_failed"
            ;;
        "errHttpAddr")
            transHtml "please_define_a_valid_http_address"
            ;;
        "mountNasSuccess")
			transHtml "mount_successfully"
            connect_success=1
            ;;
        "nasDenied")
			transHtml "mount_success_but_transmit_denied"
            ;;
        "mountNasFail")
		    transHtml "mount_failed"
            ;;
        "storeSdCardSuccess")
			transHtml "test_sd_card_successfully"
            connect_success=1
            ;;
        "storeSdCardFail")
            transHtml "test_sd_failed"
	        transHtml "the_device_is_not_existed"
	        transHtml "check_sd_card"
            ;;
        "invalidPasswdRefer")
            echo "invalid password reference"
            ;;
        *)
            ;;
    esac

    resHtmlTail
}

resJson()
{
    echo -ne "Content-type: application/json\r\n"
    echo -ne "\r\n"

    respnse_msg=$1
    case "${respnse_msg}" in
        "sendEmailSuccess" | "sendFtpSuccess" | "sendHttpSuccess" | "mountNasSuccess" | "storeSdCardSuccess")
            echo "{\"testConnection\":\"success\"}"
            connect_success=1
            ;;
        "invalidPasswdRefer")
            echo "{\"testConnection\":\"invalid password reference\"}"
            ;;
        *)
            echo "{\"testConnection\":\"fail\"}"
            ;;
    esac
}

response_from_server()
{
    response_msg="$1"
    if [ "${resType}" == "json" ]; then
        resJson $response_msg
    else
        resHtml $response_msg
    fi
}


get_qos_eventalarm_settings
get_request_parameter

connect_success=0
response_msg=$(send_request_to_server)
response_from_server ${response_msg}

exit ${connect_success}


